EXPORTS
srtp_init
srtp_shutdown
srtp_protect
srtp_protect_mki
srtp_unprotect
srtp_unprotect_mki
srtp_create
srtp_add_stream
srtp_remove_stream
srtp_update
srtp_update_stream
srtp_get_stream
srtp_crypto_policy_set_rtp_default
srtp_crypto_policy_set_rtcp_default
srtp_crypto_policy_set_aes_cm_128_hmac_sha1_32
srtp_crypto_policy_set_aes_cm_128_null_auth
srtp_crypto_policy_set_null_cipher_hmac_sha1_80
srtp_crypto_policy_set_null_cipher_hmac_null
srtp_crypto_policy_set_aes_cm_256_hmac_sha1_80
srtp_crypto_policy_set_aes_cm_256_hmac_sha1_32
srtp_crypto_policy_set_aes_cm_256_null_auth
srtp_crypto_policy_set_aes_gcm_128_8_auth
srtp_crypto_policy_set_aes_gcm_256_8_auth
srtp_crypto_policy_set_aes_gcm_128_8_only_auth
srtp_crypto_policy_set_aes_gcm_256_8_only_auth
srtp_crypto_policy_set_aes_gcm_128_16_auth
srtp_crypto_policy_set_aes_gcm_256_16_auth
srtp_dealloc
srtp_crypto_policy_set_from_profile_for_rtp
srtp_crypto_policy_set_from_profile_for_rtcp
srtp_profile_get_master_key_length
srtp_profile_get_master_salt_length
srtp_append_salt_to_key
srtp_get_protect_trailer_length
srtp_get_protect_rtcp_trailer_length
srtp_protect_rtcp
srtp_protect_rtcp_mki
srtp_unprotect_rtcp
srtp_unprotect_rtcp_mki
srtp_set_stream_roc
srtp_set_user_data
srtp_get_stream_roc
srtp_get_user_data
srtp_install_event_handler
srtp_get_version_string
srtp_get_version
srtp_set_debug_module
srtp_list_debug_modules
srtp_install_log_handler
srtp_err_report
srtp_crypto_kernel_load_debug_module
srtp_cipher_get_key_length
srtp_cipher_type_self_test
srtp_cipher_type_test
srtp_cipher_bits_per_second
srtp_cipher_type_alloc
srtp_cipher_dealloc
srtp_cipher_init
srtp_cipher_set_iv
srtp_cipher_output
srtp_cipher_encrypt
srtp_cipher_decrypt
srtp_cipher_get_tag
srtp_cipher_set_aad
srtp_replace_cipher_type
srtp_auth_get_key_length
srtp_auth_get_tag_length
srtp_auth_get_prefix_length
srtp_auth_type_self_test
srtp_auth_type_test
srtp_replace_auth_type
srtp_octet_string_hex_string
srtp_octet_string_is_eq
srtp_rdbx_get_window_size
